﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace LatLonToXYSample
{
    public partial class PlotALatitudeAndLongitudePointOnTheMap : UserControl
    {
        public PlotALatitudeAndLongitudePointOnTheMap()
        {
            InitializeComponent();
        }

        private void WpfMapLoaded(object sender, RoutedEventArgs e)
        {
            InitializeMap();
            combox.SelectedIndex = 0;

            
        }

        private void InitializeMap()
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.MapTools.MouseCoordinate.IsEnabled = true;
            

            ShapeFileFeatureLayer shapefileFeatureLayer = new ShapeFileFeatureLayer((@"..\..\Data\oklahoma_highway.shp"));
            shapefileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Red, 2, true);
            shapefileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay staticOverlay = new LayerOverlay();
            staticOverlay.Layers.Add(shapefileFeatureLayer);

            wpfMap1.Overlays.Add(staticOverlay);


            PointShape pointShape = new PointShape(-99.6787493, 35.5948678);
            InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(@"..\..\Data\United States.png");
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            inMemoryFeatureLayer.InternalFeatures.Add(new Feature(pointShape));

            LayerOverlay dynamicOverlay = new LayerOverlay();
            dynamicOverlay.Layers.Add(inMemoryFeatureLayer);

            wpfMap1.Overlays.Add(dynamicOverlay);

            wpfMap1.CurrentExtent = new RectangleShape(-99.90,35.86,-99.57,35.42);

             wpfMap1.Refresh();

        }

        private void SetMapToWGS84(RectangleShape rectangleShape)
        {
            Proj4Projection proj4 = new Proj4Projection();
            proj4.InternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(3501);
            proj4.ExternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();

            proj4.Open();

            if (wpfMap1.MapUnit == GeographyUnit.DecimalDegree)
            {
                wpfMap1.CurrentExtent = rectangleShape;
            }
            else
            {
                RectangleShape projRectangleShape = proj4.ConvertToExternalProjection(rectangleShape);
                rectangleShape = proj4.ConvertToExternalProjection(rectangleShape);

                //wpfMap1.CurrentExtent = projRectangleShape;
            }

            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.MapTools.MouseCoordinate.MouseCoordinateType = MouseCoordinateType.DegreesMinutesSeconds;

            foreach (LayerOverlay layerOverlay in wpfMap1.Overlays)
            {
                foreach (FeatureLayer featureLayer in layerOverlay.Layers)
                {
                    featureLayer.FeatureSource.Projection = null;
                }
            }

            wpfMap1.CurrentExtent = rectangleShape;

        }


        private void SetMapToStatePlane(RectangleShape rectangleShape)
        {
            wpfMap1.MapUnit = GeographyUnit.Feet;
            wpfMap1.MapTools.MouseCoordinate.MouseCoordinateType = MouseCoordinateType.Default;

            Proj4Projection proj4 = new Proj4Projection();
            proj4.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4.ExternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(3501);

            proj4.Open();
           
            foreach (LayerOverlay layerOverlay in wpfMap1.Overlays)
            {
                foreach (FeatureLayer featureLayer in layerOverlay.Layers)
                {
                    featureLayer.FeatureSource.Projection = proj4;
                }
            }

           
            RectangleShape projRectangleShape = proj4.ConvertToExternalProjection(rectangleShape);
          

            wpfMap1.CurrentExtent = projRectangleShape;

           

        }
      
        
        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListBoxItem lbItem = combox.SelectedItem as ListBoxItem;
            if (lbItem != null) MapCoordinates = lbItem.Content.ToString();

            if (MapCoordinates == "Lat/Lon")
            {
                SetMapToWGS84(wpfMap1.CurrentExtent);
            }
            else 
            {
                SetMapToStatePlane(wpfMap1.CurrentExtent);
            }
            
            wpfMap1.Refresh();

        }

        public string MapCoordinates { get; set; }
    }
}
